<?php

namespace App\Http;

use Illuminate\Foundation\Http\Kernel as HttpKernel;

class Kernel extends HttpKernel
{
    /**
     * Global HTTP middleware stack.
     *
     * These middleware are run during every request to your application.
     */
    protected $middleware = [];

    /**
     * The application's route middleware groups.
     */
    protected $middlewareGroups = [
        'web' => [
            \App\Http\Middleware\BlockDemoAccess::class
        ],

        'api' => [
            \App\Http\Middleware\CheckProductStatus::class,
            \App\Http\Middleware\BlockDemoAccess::class,
        ],
    ];

    /**
     * The application's route middleware.
     *
     * These can be assigned to groups or used individually.
     */
    protected $routeMiddleware = [
        'add.org_id' => \App\Http\Middleware\AddOrgId::class,
        'blockDemoAccess' => \App\Http\Middleware\BlockDemoAccess::class,

    ];
}
